using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Diagnostics;

namespace Autorun
{
	[Serializable]
	public sealed class MonitorManagement : MarshalByRefObject
	{
		#region "Private member"
		private Timer myTimer;
		private int[,] rawData = new int[5, 86401]; // I use the indices [1..86400]
		private int resX; // 1 px = ? seconds --> value included in ]0:180]
		private PerformanceCounter myCPUCounter;
		private int lastOffset;
		private int processorCount;
		#endregion

		#region "Singleton Design-Pattern"
		private static MonitorManagement _instance = new MonitorManagement();

		public static MonitorManagement Instance
		{
			get { return _instance; }
		}

		private MonitorManagement()
		{
			resX = 1; // Bu default: 1px = 1sec
			rawData[(int)Counters.ProcessorUsage, 86400] = 100; // Collected values are already in %

			//http://blogs.msdn.com/bclteam/archive/2006/09/08/746900.aspx
			// Open the Registry Editor by going to the Start Menu and selecting Run, then type regedit, and click the OK button. 
			// Navigate to the HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\SecurePipeServers\winreg registry key. 
			// Right click on the "winreg" key and select Permissions. Add users or groups to which you want to grant Read access. 
			// Exit Registry Editor and restart Windows. 
			//
			// For more explanation on this process see http://support.microsoft.com/?kbid=153183.
			myCPUCounter = new PerformanceCounter("Process", "% Processor Time", AppDomain.CurrentDomain.FriendlyName.Replace(".exe", ""));
			processorCount = Environment.ProcessorCount;
		}

		~MonitorManagement()
		{
			if (myTimer != null) myTimer.Dispose();
		}
		#endregion

		#region "Properties"
		//
		public int Resolution
		{
			get { return resX; }
			set
			{
				if ((value > 0) && (value < 181))
					resX = value;
				else
					throw new OverflowException(String.Format("Invalid resolution value '{0}': must be included in ]0:180]"));
			}
		}

		public int[,] Data
		{
			get
			{
				int[,] rc = new int[4, 480];
				int offset;
				int grp;
				int value;
				int total;

				// ====================================================================
				// === PRE-PROCESS                                                  ===
				// ====================================================================
				total = 0;
				// --- Calculer l'offset de dpart dans rawData -----------------------
				offset = (int)DateTime.Now.TimeOfDay.TotalSeconds;
				offset = offset - (offset % resX);

				// ====================================================================
				// === PROCESS                                                      ===
				// ====================================================================
				// --- Remplir les colonnes du tableau --------------------------------
				for (int col = 0; col < 480; col++)
				{
					// --- Pour chaque colonne, remplir les lignes du tableau -----------
					// --- Pour chaque cellule, faire le total du groupage ------------
					grp = 0;
					value = 0;
					while (grp < resX)
					{
						value += rawData[(int)Counters.ProcessorUsage, offset];
						if (offset == 0) offset = 86400;
						offset -= 1;// on remonte dans le temps
						grp += 1;
					}
					// --- Faire la moyenne du groupage -------------------------------
					// Sauver la valeur en %
					if (value == 0)
						rc[(int)Counters.ProcessorUsage, col] = 0;
					else
						if (rawData[(int)Counters.ProcessorUsage, 86400] > 0)
						{
							value = ((value / resX) * 100) / rawData[(int)Counters.ProcessorUsage, 86400];
							rc[(int)Counters.ProcessorUsage, col] = value;
						}

					total = total + rc[(int)Counters.ProcessorUsage, col];
					rc[(int)Counters.ProcessorAverage, col] = total / (col + 1);
				}

				total = 50;
				for (int col = 479; col > -1; col--)
				{
					total = total + rc[(int)Counters.ProcessorUsage, col];
					rc[(int)Counters.ProcessorAverage, col] = total / (480 - col);
				}
				return rc;
			}
		}
		#endregion

		#region "Procedures and Functions"
		internal void Start()
		{
			lastOffset = (int)DateTime.Now.TimeOfDay.TotalSeconds - 1;
			myTimer = new Timer(CollectData, null, 0, 1000);
		}

		internal void Stop()
		{
			if (myTimer != null)
			{
				myTimer.Dispose();
				myTimer = null;
			}
		}

		void CollectData(object state)
		{
			int offset;
			int dx;

			// ======================================================================
			// === PRE-PROCESS                                                    ===
			// ======================================================================
			// --- Calculer l'offset en fonction de l'heure actuelle ----------------
			offset = (int)DateTime.Now.TimeOfDay.TotalSeconds;
			// ======================================================================
			// === PROCESS                                                        ===
			// ======================================================================
			// --- 1. Processor -----------------------------------------------------
			//      rawData(Counters.Processor, offset) = myCPUCounter.RawValue
			if (myCPUCounter != null) rawData[(int)Counters.ProcessorUsage, offset] = (int)myCPUCounter.NextValue() / processorCount;
			//	if (rawData[(int)Counters.Processor, offset] > 100) Debug.Fail("Value > 100%");
			//If rawData(Counters.Processor, offset) > rawData(Counters.Processor, 86400) Then
			//  rawData(Counters.Processor, 86400) = rawData(Counters.Processor, offset)
			//End If

			// --- 2. Memory --------------------------------------------------------

			// --- 3. Active Tasks --------------------------------------------------
			//rawData(Counters.RunningServices, offset) = My.RunningServices.Count
			//If rawData(Counters.RunningServices, offset) > rawData(Counters.RunningServices, 86400) Then
			//  rawData(Counters.RunningServices, 86400) = rawData(Counters.RunningServices, offset)
			//End If
			// --- 4. Waiting Tasks -------------------------------------------------
			//rawData(Counters.WaitingServices, offset) = My.WaitingServices.Count
			//If rawData(Counters.WaitingServices, offset) > rawData(Counters.WaitingServices, 86400) Then
			//  rawData(Counters.WaitingServices, 86400) = rawData(Counters.WaitingServices, offset)
			//End If

			if (lastOffset + 1 != offset)
			{
				dx = rawData[(int)Counters.ProcessorUsage, offset] - rawData[(int)Counters.ProcessorUsage, lastOffset];
				lastOffset++;
				while (lastOffset < offset)
				{
					rawData[(int)Counters.ProcessorUsage, lastOffset] = rawData[(int)Counters.ProcessorUsage, lastOffset - 1] + dx;
					lastOffset++;
				}
			}
			lastOffset = offset;
		}


		#endregion
	}
}
